@@ -3,11 +3,11 @@
 #  augment_defaults.py
 """
 Sphinx's autodoc module allows for default options to be set,
-and allows for those defaults to be disabled for an auto* directive and different values given instead.
+and allows for those defaults to be disabled for an ``:auto*:`` directive
+and different values given instead.
 
 However, it does not appear to be possible to augment the defaults,
 such as to globally exclude certain members and then exclude additional members of a single class.
-
 This module monkeypatches in that behaviour.
 
 .. extensions:: sphinx_toolbox.more_autodoc.augment_defaults
@@ -15,9 +15,9 @@ This module monkeypatches in that behaviour.
 .. versionchanged:: 0.6.0
 
 	Moved from :mod:`sphinx_toolbox.autodoc_augment_defaults`.
-"""  # noqa D400
+"""  # noqa: D400
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -66,21 +66,22 @@ This module monkeypatches in that behaviour.
 #
 
 # stdlib
-from typing import Any, Dict, List, Type
+from typing import Dict, List, Type
 
 # 3rd party
+import autodocsumm  # type: ignore[import-untyped]
 import sphinx.ext.autodoc.directive
 from docutils.utils import assemble_option_dict
 from sphinx.application import Sphinx
-from sphinx.config import Config
 from sphinx.errors import ExtensionError
 from sphinx.ext.autodoc import Documenter, Options
 
 # this package
 import sphinx_toolbox
-from sphinx_toolbox.utils import SphinxExtMetadata
+import sphinx_toolbox.more_autosummary
+from sphinx_toolbox.utils import Config, SphinxExtMetadata
 
-__all__ = ["process_documenter_options", "setup"]
+__all__ = ("process_documenter_options", "setup")
 
 
 def process_documenter_options(
@@ -103,6 +104,7 @@ def process_documenter_options(
 			negated = options.pop("no-" + name, True) is None
 
 			if name in config.autodoc_default_options and not negated:
+				# pylint: disable=loop-invariant-statement
 				default_value = config.autodoc_default_options[name]
 				existing_value = options.get(name, None)
 				values: List[str] = [v for v in [default_value, existing_value] if v not in {None, True, False}]
@@ -111,6 +113,7 @@ def process_documenter_options(
 					options[name] = ','.join(values)
 				else:
 					options[name] = None  # pragma: no cover
+				# pylint: enable=loop-invariant-statement
 
 	return Options(assemble_option_dict(options.items(), documenter.option_spec))
 
@@ -119,15 +122,18 @@ def setup(app: Sphinx) -> SphinxExtMetadata:
 	"""
 	Setup :mod:`sphinx_toolbox.more_autodoc.augment_defaults`.
 
-	:param app: The Sphinx app.
+	:param app: The Sphinx application.
 	"""
 
 	if "sphinx.ext.autodoc" in app.extensions:
 		raise ExtensionError(
-				"'sphinx_toolbox.more_autodoc.augment_defaults' must be loaded before 'sphinx.ext.autodoc."
+				"'sphinx_toolbox.more_autodoc.augment_defaults' "
+				"must be loaded before 'sphinx.ext.autodoc'."
 				)
 
-	sphinx.ext.autodoc.directive.process_documenter_options = process_documenter_options
+	sphinx.ext.autodoc.directive.process_documenter_options = process_documenter_options  # type: ignore[assignment]
+	autodocsumm.process_documenter_options = process_documenter_options
+	sphinx_toolbox.more_autosummary.process_documenter_options = process_documenter_options  # type: ignore[assignment]
 
 	app.setup_extension("sphinx.ext.autodoc")
 
