@@ -2,20 +2,20 @@
 #
 #  genericalias.py
 """
-Documenter for alias, which usually manifest as
-`type aliases <https://docs.python.org/3/library/typing.html#type-aliases>`_.
+Documenter for alias, which usually manifest as `type aliases`_.
 
+.. _type aliases: https://docs.python.org/3/library/typing.html#type-aliases
+
+.. versionadded:: 0.6.0
 .. extensions:: sphinx_toolbox.more_autodoc.genericalias
 
 .. note::
 
 	:mod:`sphinx_toolbox.more_autodoc.genericalias` is only supported on Python 3.7 and above.
 
-.. versionadded:: 0.6.0
-
-"""  # noqa D400
+"""
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -65,32 +65,51 @@ Documenter for alias, which usually manifest as
 #
 
 # stdlib
-from typing import Any, Dict
+from typing import Any
 
 # 3rd party
 from docutils.statemachine import StringList
 from sphinx.application import Sphinx
-from sphinx.ext.autodoc import DataDocumenter, GenericAliasDocumenter
-from sphinx.ext.autodoc.directive import DocumenterBridge
+from sphinx.ext.autodoc import SUPPRESS, Options
+from sphinx.ext.autodoc.directive import DocumenterBridge  # noqa: F401
 from sphinx.locale import _
+from sphinx.util import inspect
 
 # this package
-from sphinx_toolbox import __version__
+from sphinx_toolbox._data_documenter import DataDocumenter
 from sphinx_toolbox.more_autodoc.typehints import format_annotation
-from sphinx_toolbox.utils import SphinxExtMetadata
+from sphinx_toolbox.utils import SphinxExtMetadata, metadata_add_version
 
-__all__ = ["PrettyGenericAliasDocumenter", "setup"]
+__all__ = ("PrettyGenericAliasDocumenter", "setup")
 
 
-class PrettyGenericAliasDocumenter(GenericAliasDocumenter):  # pragma: no cover (<Py37)
+class PrettyGenericAliasDocumenter(DataDocumenter):
+	"""
+	Specialized Documenter subclass for GenericAliases, with prettier output than Sphinx's one.
 	"""
-	Specialized Documenter subclass for GenericAliases,
-	with prettier output than Sphinx's one.
-	"""  # noqa D400
 
-	priority = GenericAliasDocumenter.priority + 1
+	objtype = "genericalias"
+	directivetype = "data"
+	priority = DataDocumenter.priority + 2
 
-	def add_content(self, more_content: Any, no_docstring: bool = False):
+	@classmethod
+	def can_document_member(cls, member: Any, membername: str, isattr: bool, parent: Any) -> bool:
+		"""
+		Called to see if a member can be documented by this documenter.
+		"""
+
+		return inspect.isgenericalias(member)
+
+	def add_directive_header(self, sig: str) -> None:
+		"""
+		Add the directive header and options to the generated content.
+		"""
+
+		self.options = Options(self.options)
+		self.options["annotation"] = SUPPRESS
+		super().add_directive_header(sig)
+
+	def add_content(self, more_content: Any, no_docstring: bool = False) -> None:
 		"""
 		Add the autodocumenter content.
 
@@ -103,17 +122,15 @@ class PrettyGenericAliasDocumenter(GenericAliasDocumenter):  # pragma: no cover
 		DataDocumenter.add_content(self, content)
 
 
+@metadata_add_version
 def setup(app: Sphinx) -> SphinxExtMetadata:
 	"""
 	Setup :mod:`sphinx_toolbox.more_autodoc.genericalias`.
 
-	:param app: The Sphinx app.
+	:param app: The Sphinx application.
 	"""
 
 	app.setup_extension("sphinx.ext.autodoc")
 	app.add_autodocumenter(PrettyGenericAliasDocumenter, override=True)
 
-	return {
-			"version": __version__,
-			"parallel_read_safe": True,
-			}
+	return {"parallel_read_safe": True}
