@@ -182,6 +182,9 @@ except ImportError:
 	from sphinx_autodoc_typehints import _LOGGER as sat_logger
 	from sphinx_autodoc_typehints import _PYDATA_ANNOTATIONS as pydata_annotations
 
+if isinstance(next(iter(pydata_annotations)), tuple):
+	pydata_annotations = {x[1] for x in pydata_annotations}
+
 try:
 	# 3rd party
 	from sphinx_autodoc_typehints import _future_annotations_imported
@@ -372,6 +375,11 @@ def format_annotation(annotation: Any, fully_qualified: bool = False) -> str:
 
 		args = get_annotation_args(annotation, module, class_name)
 	except ValueError:
+		if isinstance(annotation, str) and '|' in annotation and sys.version_info < (3, 10):
+			sat_logger.warning(
+					"Unsupported | character in type annotation '%s' for this version of Python",
+					annotation,
+					)
 		return f":py:obj:`~.{annotation}`"
 
 	if module == "_io":
@@ -404,7 +412,6 @@ def format_annotation(annotation: Any, fully_qualified: bool = False) -> str:
 		full_name = "typing.Optional"
 	elif full_name == "types.UnionType":
 		full_name = "typing.Union"
-		role = "data"
 	elif full_name == "typing.Callable" and args and args[0] is not ...:
 		formatted_args = "\\[\\[" + ", ".join(format_annotation(arg) for arg in args[:-1]) + ']'
 		formatted_args += ", " + format_annotation(args[-1]) + ']'
@@ -419,6 +426,9 @@ def format_annotation(annotation: Any, fully_qualified: bool = False) -> str:
 
 		formatted_args = f"\\[{formatted_arg_list:, }]"
 
+	if full_name == "typing.Union":
+		role = "class"  # With at least the 3.14 docs.
+
 	if full_name == "typing.Optional":
 		args = tuple(x for x in args if x is not type(None))  # noqa: E721
 
@@ -924,4 +934,4 @@ def _resolve_forwardref(
 	elif sys.version_info >= (3, 12):
 		return fr._evaluate(module_dict, module_dict, set(), recursive_guard=set())
 	else:
-		return fr._evaluate(module_dict, module_dict, set())
+		return fr._evaluate(module_dict, module_dict, set())  # type: ignore[arg-type]
