@@ -3,35 +3,15 @@
 #  typevars.py
 r"""
 Documenter for module level :class:`typing.TypeVar`\'s, similar to Sphinx's
-:rst:dir:`sphinx:autotypevar` but with a different appearance.
-
-.. extensions:: sphinx_toolbox.more_autodoc.typevars
+``autotypevar`` but with a different appearance.
 
 .. versionadded:: 1.3.0
+.. extensions:: sphinx_toolbox.more_autodoc.typevars
 
-Usage
-----------
-
-.. rst:directive:: autotypevar
-
-	Directive to automatically document a :class:`typing.TypeVar`.
-
-	The output is based on the :rst:dir:`autodata` directive, and takes all of its options,
-	plus these additional ones:
-
-	.. rst:directive:option:: no-value
-
-		Don't show the value of the variable.
-
-	.. rst:directive:option:: value: value
-		:type: string
-
-		Show this instead of the value taken from the Python source code.
-
-	.. rst:directive:option:: no-type
-
-		Don't show the type of the variable.
+.. latex:vspace:: -15px
 
+Configuration
+-------------
 
 .. confval:: all_typevars
 	:type: :class:`bool`
@@ -48,11 +28,38 @@ Usage
 
 	This option has no effect if :confval:`all_typevars` is False.
 
+
+.. latex:clearpage::
+
+Usage
+----------
+
+.. rst:directive:: autotypevar
+
+	Directive to automatically document a :class:`typing.TypeVar`.
+
+	The output is based on the :rst:dir:`autodata` directive, and takes all of its options
+	plus these additional ones:
+
+	.. rst:directive:option:: no-value
+
+		Don't show the value of the variable.
+
+	.. rst:directive:option:: value: value
+		:type: string
+
+		Show this instead of the value taken from the Python source code.
+
+	.. rst:directive:option:: no-type
+
+		Don't show the type of the variable.
+
+
 API Reference
 ----------------
-"""  # noqa D400
+"""  # noqa: D400
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -103,37 +110,35 @@ API Reference
 
 # stdlib
 import sys
-from types import ModuleType
-from typing import Any, Dict, List, Optional, Tuple, Type, TypeVar, Union
+from typing import Any, Dict, ForwardRef, List, Optional, Tuple, Type, TypeVar, Union
 
 # 3rd party
+from docutils.statemachine import StringList
 from domdf_python_tools.words import word_join
 from sphinx.application import Sphinx
-from sphinx.ext.autodoc import DataDocumenter
 from typing_extensions import Protocol
 
 # this package
-from sphinx_toolbox import __version__
+from sphinx_toolbox._data_documenter import DataDocumenter
 from sphinx_toolbox.config import ToolboxConfig
-from sphinx_toolbox.more_autodoc.typehints import ForwardRef, format_annotation
+from sphinx_toolbox.more_autodoc import _documenter_add_content
+from sphinx_toolbox.more_autodoc.typehints import format_annotation
 from sphinx_toolbox.more_autodoc.variables import VariableDocumenter
-from sphinx_toolbox.utils import SphinxExtMetadata
+from sphinx_toolbox.utils import SphinxExtMetadata, metadata_add_version
 
-__all__ = [
+__all__ = (
 		"TypeVarDocumenter",
 		"unskip_typevars",
 		"setup",
-		]
+		)
 
 
 class TypeVarDocumenter(VariableDocumenter):
-	"""
+	r"""
 	Alternative version of :class:`sphinx.ext.autodoc.TypeVarDocumenter`
 	with better type hint rendering.
 
-	Specialized Documenter subclass for :class:`typing.TypeVars`.
-
-	.. versionadded:: 1.3.0
+	Specialized Documenter subclass for :class:`typing.TypeVar`\s.
 	"""  # noqa: D400
 
 	objtype = "typevar"
@@ -159,7 +164,7 @@ class TypeVarDocumenter(VariableDocumenter):
 
 		return isinstance(member, TypeVar)
 
-	def resolve_type(self, forward_ref: ForwardRef) -> Type:
+	def resolve_type(self, forward_ref: "ForwardRef") -> Type:
 		"""
 		Resolve a :class:`typing.ForwardRef` using the module the :class:`~typing.TypeVar` belongs to.
 
@@ -167,20 +172,14 @@ class TypeVarDocumenter(VariableDocumenter):
 		"""
 
 		if forward_ref.__forward_evaluated__:
-			return forward_ref.__forward_value__
+			return forward_ref.__forward_value__  # type: ignore[return-value]
 		else:
-			if sys.version_info[:2] == (3, 6) and self.object.__module__ == "typing" and isinstance(
-					self.parent, ModuleType
-					):
-				# __module__ is 'typing' for 3.6
-				globanls = self.parent.__dict__
-			else:
-				globanls = sys.modules[self.object.__module__].__dict__
+			globanls = sys.modules[self.object.__module__].__dict__
 
 			eval_ = eval
 			return eval_(forward_ref.__forward_code__, globanls, globanls)
 
-	def add_content(self, more_content: Any, no_docstring: bool = False) -> None:
+	def add_content(self, more_content: Optional[StringList], no_docstring: bool = False) -> None:
 		"""
 		Add content from docstrings, attribute documentation and user.
 
@@ -227,7 +226,7 @@ class TypeVarDocumenter(VariableDocumenter):
 		self.add_line(' '.join(description).rstrip() + '.', sourcename)  # "   " +
 		self.add_line('', sourcename)
 
-		super().add_content(more_content, no_docstring)
+		_documenter_add_content(self, more_content, no_docstring)
 
 	def add_directive_header(self, sig: str) -> None:
 		"""
@@ -264,31 +263,37 @@ class TypeVarDocumenter(VariableDocumenter):
 
 		super().add_directive_header(sig)
 
-	def get_doc(self, encoding: str = None, ignore: int = None) -> List[List[str]]:
+	def get_doc(
+			self,
+			encoding: Optional[str] = None,
+			ignore: Optional[int] = None,
+			) -> List[List[str]]:
 		"""
 		Decode and return lines of the docstring(s) for the object.
 
 		:param encoding:
 		:param ignore:
+
+		:rtype:
+
+		.. latex:clearpage::
 		"""
 
 		if self.object.__doc__ != TypeVar.__doc__:
-			return super().get_doc()
+			return super().get_doc() or []
 		else:
 			return []
 
 
-def validate_config(app: Sphinx, config: ToolboxConfig):
+def validate_config(app: Sphinx, config: ToolboxConfig) -> None:
 	r"""
 	Validate the provided configuration values.
 
 	See :class:`~sphinx_toolbox.config.ToolboxConfig` for a list of the configuration values.
 
-	:param app: The Sphinx app.
+	:param app: The Sphinx application.
 	:param config:
 	:type config: :class:`~sphinx.config.Config`
-
-	.. versionadded:: 1.3.0
 	"""
 
 	if config.all_typevars:
@@ -306,7 +311,7 @@ def unskip_typevars(
 	r"""
 	Unskip undocumented :class:`typing.TypeVar`\s if :confval:`all_typevars` is :py:obj:`True`.
 
-	:param app: The Sphinx application object.
+	:param app: The Sphinx application.
 	:param what: The type of the object which the docstring belongs to (one of
 		``'module'``, ``'class'``, ``'exception'``, ``'function'``, ``'method'``,
 		``'attribute'``).
@@ -318,10 +323,10 @@ def unskip_typevars(
 		``inherited_members``, ``undoc_members``, ``show_inheritance`` and
 		``noindex`` that are true if the flag option of same name was given to the
 		auto directive.
-
-	.. versionadded:: 1.3.0
 	"""
 
+	assert app.env is not None
+
 	if isinstance(obj, TypeVar):
 		if app.env.config.no_unbound_typevars:
 			if obj.__bound__ or obj.__constraints__:
@@ -334,13 +339,12 @@ def unskip_typevars(
 	return None
 
 
+@metadata_add_version
 def setup(app: Sphinx) -> SphinxExtMetadata:
 	"""
 	Setup :mod:`sphinx_toolbox.more_autodoc.typevars`.
 
-	:param app: The Sphinx app.
-
-	.. versionadded:: 1.3.0
+	:param app: The Sphinx application.
 	"""
 
 	app.setup_extension("sphinx.ext.autodoc")
@@ -350,20 +354,12 @@ def setup(app: Sphinx) -> SphinxExtMetadata:
 
 	app.connect("config-inited", validate_config, priority=850)
 
-	return {
-			"version": __version__,
-			"parallel_read_safe": True,
-			}
+	return {"parallel_read_safe": True}
 
 
 class _TypeVar(Protocol):
-	if sys.version_info < (3, 7):
-		__constraints__: Tuple[Any, ...]
-		__bound__: Union[Type, Any, None]
-	else:
-		__constraints__: Tuple[Union[Type, ForwardRef], ...]
-		__bound__: Union[Type, ForwardRef, None]
-
+	__constraints__: Tuple[Union[Type, ForwardRef], ...]
+	__bound__: Union[Type, ForwardRef, None]
 	__covariant__: bool
 	__contravariant__: bool
 	__name__: str
