@@ -4,9 +4,8 @@
 """
 Sphinx extension to show examples of ``.pre-commit-config.yaml`` configuration.
 
-.. extensions:: sphinx_toolbox.pre_commit
-
 .. versionadded:: 1.6.0
+.. extensions:: sphinx_toolbox.pre_commit
 
 
 Usage
@@ -14,7 +13,7 @@ Usage
 
 .. rst:directive:: pre-commit
 
-	Shows an example snippet of ``.pre-commit-config.yaml``.
+	Directive which shows an example snippet of ``.pre-commit-config.yaml``.
 
 	.. rst:directive:option:: rev
 		:type: string
@@ -36,7 +35,7 @@ Usage
 		.. versionadded:: 1.7.2
 
 
-	**Example**
+	:bold-title:`Example`
 
 	.. rest-example::
 
@@ -44,10 +43,11 @@ Usage
 			:rev: v0.0.4
 			:hooks: some-hook,some-other-hook
 
+.. clearpage::
 
-.. rst:directive:: pre-commit:flake8
+.. rst:directive:: .. pre-commit:flake8:: version
 
-	Shows an example snippet of ``.pre-commit-config.yaml`` for a flake8 plugin.
+	Directive which shows an example snippet of ``.pre-commit-config.yaml`` for a flake8 plugin.
 
 	The directive takes a single argument -- the version of the flake8 plugin to install from PyPI.
 
@@ -61,18 +61,19 @@ Usage
 
 		The name of the plugin to install from PyPI. Defaults to the repository name.
 
-	**Example**
+	:bold-title:`Example`
 
 	.. rest-example::
 
 		.. pre-commit:flake8:: 0.0.4
 
+	.. versionchanged:: 2.8.0  The repository URL now points to GitHub.
 
 API Reference
 ----------------
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -96,14 +97,16 @@ API Reference
 # stdlib
 import re
 import warnings
+from io import StringIO
 from textwrap import indent
-from typing import List, Sequence
+from typing import Any, List, Sequence
 
 # 3rd party
-import ruamel.yaml as yaml
+import sphinx.util.docutils
 from docutils import nodes
 from docutils.statemachine import StringList
 from domdf_python_tools.paths import PathPlus
+from ruamel.yaml import YAML
 from sphinx.application import Sphinx
 from sphinx.util.docutils import SphinxDirective
 from typing_extensions import TypedDict
@@ -111,14 +114,14 @@ from typing_extensions import TypedDict
 # this package
 from sphinx_toolbox.utils import Purger, SphinxExtMetadata, make_github_url, metadata_add_version
 
-__all__ = [
+__all__ = (
 		"pre_commit_node_purger",
 		"pre_commit_f8_node_purger",
 		"parse_hooks",
 		"PreCommitDirective",
 		"Flake8PreCommitDirective",
 		"setup",
-		]
+		)
 
 pre_commit_node_purger = Purger("all_pre_commit_nodes")
 pre_commit_f8_node_purger = Purger("all_pre_commit_f8_nodes")
@@ -154,6 +157,8 @@ class _Config(_BaseConfig, total=False):
 class PreCommitDirective(SphinxDirective):
 	"""
 	A Sphinx directive for documenting pre-commit hooks.
+
+	.. clearpage::
 	"""
 
 	has_content: bool = False
@@ -163,7 +168,7 @@ class PreCommitDirective(SphinxDirective):
 			"args": parse_hooks,
 			}
 
-	def run(self) -> Sequence[nodes.Node]:  # type: ignore
+	def run(self) -> Sequence[nodes.Node]:  # type: ignore[override]
 		"""
 		Process the content of the directive.
 		"""
@@ -174,8 +179,10 @@ class PreCommitDirective(SphinxDirective):
 			cwd = PathPlus.cwd()
 
 			for directory in (cwd, *cwd.parents):
-				if (directory / ".pre-commit-hooks.yaml").is_file():
-					hooks = [h["id"] for h in yaml.safe_load((directory / ".pre-commit-hooks.yaml").read_text())]
+				hook_file = directory / ".pre-commit-hooks.yaml"
+				if hook_file.is_file():
+					hooks_dict = YAML(typ="safe", pure=True).load(hook_file.read_text())
+					hooks = [h["id"] for h in hooks_dict]  # pylint: disable=loop-invariant-statement
 					break
 			else:
 				warnings.warn("No hooks specified and no .pre-commit-hooks.yaml file found.")
@@ -195,7 +202,14 @@ class PreCommitDirective(SphinxDirective):
 		targetid = f'pre-commit-{self.env.new_serialno("pre-commit"):d}'
 		targetnode = nodes.section(ids=[targetid])
 
-		yaml_output = yaml.round_trip_dump([config], default_flow_style=False)
+		yaml_dumper = YAML()
+		yaml_dumper.default_flow_style = False
+
+		yaml_output_stream = StringIO()
+		yaml_dumper.dump([config], stream=yaml_output_stream)
+
+		yaml_output = yaml_output_stream.getvalue()
+
 		if not yaml_output:
 			return []
 
@@ -221,7 +235,7 @@ class Flake8PreCommitDirective(SphinxDirective):
 			}
 	required_arguments = 1  # the plugin version
 
-	def run(self) -> Sequence[nodes.Node]:  # type: ignore
+	def run(self) -> Sequence[nodes.Node]:  # type: ignore[override]
 		"""
 		Process the content of the directive.
 		"""
@@ -230,7 +244,7 @@ class Flake8PreCommitDirective(SphinxDirective):
 		flake8_version = self.options.get("flake8-version", "3.8.4")
 
 		config = {
-				"repo": "https://gitlab.com/pycqa/flake8",
+				"repo": "https://github.com/pycqa/flake8",
 				"rev": flake8_version,
 				"hooks": [{"id": "flake8", "additional_dependencies": [f"{plugin_name}=={self.arguments[0]}"]}]
 				}
@@ -238,7 +252,14 @@ class Flake8PreCommitDirective(SphinxDirective):
 		targetid = f'pre-commit-{self.env.new_serialno("pre-commit"):d}'
 		targetnode = nodes.section(ids=[targetid])
 
-		yaml_output = yaml.round_trip_dump([config], default_flow_style=False)
+		yaml_dumper = YAML()
+		yaml_dumper.default_flow_style = False
+
+		yaml_output_stream = StringIO()
+		yaml_dumper.dump([config], stream=yaml_output_stream)
+
+		yaml_output = yaml_output_stream.getvalue()
+
 		if not yaml_output:
 			return []
 
@@ -252,12 +273,76 @@ class Flake8PreCommitDirective(SphinxDirective):
 		return [pre_commit_node]
 
 
+def revert_8345() -> None:  # pragma: no cover  # Only for Sphinx 4+
+	"""
+	Remove the incorrect warning emitted since https://github.com/sphinx-doc/sphinx/pull/8345.
+	"""
+
+	#  Copyright (c) 2007-2020 by the Sphinx team (see AUTHORS file).
+	#  BSD Licensed
+	#  All rights reserved.
+	#
+	#  Redistribution and use in source and binary forms, with or without
+	#  modification, are permitted provided that the following conditions are
+	#  met:
+	#
+	#  * Redistributions of source code must retain the above copyright
+	#   notice, this list of conditions and the following disclaimer.
+	#
+	#  * Redistributions in binary form must reproduce the above copyright
+	#   notice, this list of conditions and the following disclaimer in the
+	#   documentation and/or other materials provided with the distribution.
+	#
+	#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+	#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+	#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+	#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+	#  HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+	#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+	#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+	#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+	#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+	#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+	#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+	def lookup_domain_element(self, type: str, name: str) -> Any:  # noqa: A002  # pylint: disable=redefined-builtin
+		"""
+		Lookup a markup element (directive or role), given its name which can be a full name (with domain).
+		"""
+
+		name = name.lower()
+		# explicit domain given?
+		if ':' in name:
+			domain_name, name = name.split(':', 1)
+			if domain_name in self.env.domains:
+				domain = self.env.get_domain(domain_name)
+				element = getattr(domain, type)(name)
+				if element is not None:
+					return element, []
+		# else look in the default domain
+		else:
+			def_domain = self.env.temp_data.get("default_domain")
+			if def_domain is not None:
+				element = getattr(def_domain, type)(name)
+				if element is not None:
+					return element, []
+
+		# always look in the std domain
+		element = getattr(self.env.get_domain("std"), type)(name)
+		if element is not None:
+			return element, []
+
+		raise sphinx.util.docutils.ElementLookupError
+
+	sphinx.util.docutils.sphinx_domains.lookup_domain_element = lookup_domain_element  # type: ignore[method-assign]
+
+
 @metadata_add_version
 def setup(app: Sphinx) -> SphinxExtMetadata:
 	"""
 	Setup :mod:`sphinx_toolbox.pre_commit`.
 
-	:param app: The Sphinx app.
+	:param app: The Sphinx application.
 	"""
 
 	app.add_directive("pre-commit", PreCommitDirective)
@@ -265,4 +350,7 @@ def setup(app: Sphinx) -> SphinxExtMetadata:
 	app.connect("env-purge-doc", pre_commit_node_purger.purge_nodes)
 	app.connect("env-purge-doc", pre_commit_f8_node_purger.purge_nodes)
 
+	if sphinx.version_info >= (4, 0):
+		revert_8345()
+
 	return {"parallel_read_safe": True}
