@@ -237,33 +237,7 @@ Shields
 		.. rest-example::
 
 			.. actions-shield::
-				:workflow: Windows Tests
-
-
-.. rst:directive:: requires-io-shield
-
-	Shield to show the *Requires.io* status.
-
-	.. rst:directive:option:: username
-
-		The GitHub username. Defaults to :confval:`github_username`.
-
-	.. rst:directive:option:: repository
-
-		The GitHub repository. Defaults to :confval:`github_repository`.
-
-	.. rst:directive:option:: branch
-
-		The branch to show the build status for. Default ``master``.
-
-
-	.. only:: html
-
-		**Example**
-
-		.. rest-example::
-
-			.. requires-io-shield::
+				:workflow: Windows
 
 
 .. rst:directive:: coveralls-shield
@@ -383,6 +357,7 @@ Shields
 #
 
 # stdlib
+import warnings
 from typing import List, Optional, Tuple
 from urllib.parse import quote
 
@@ -592,6 +567,8 @@ class GitHubActionsShield(GitHubBackedShield):
 class RequiresIOShield(GitHubBackedShield):
 	"""
 	Shield to show the `Requires.io <https://requires.io>`_ status.
+
+	.. deprecated:: The ``requires.io`` service has shut down.
 	"""
 
 	option_spec: OptionSpec = {
@@ -606,6 +583,11 @@ class RequiresIOShield(GitHubBackedShield):
 		Process the content of the shield directive.
 		"""
 
+		warnings.warn(
+				"RequiresIOShield (.. requires-io-shield::) is deprecated as requires.io has shut down",
+				DeprecationWarning
+				)
+
 		username, repository = self.get_repo_details()
 		branch = self.options.pop("branch", "master")
 		base_url = URL("https://requires.io/github/") / username / repository
