@@ -761,8 +761,8 @@ class HTMLRegressionFixture(FileRegressionFixture):
 				if div.string == '=':
 					sibling = div.next_sibling
 					assert sibling is not None
-					div.replace_with('')  # type: ignore[arg-type]
-					sibling.replace_with(f"= {sibling.text}")  # type: ignore[arg-type]
+					div.replace_with('')
+					sibling.replace_with(f"= {sibling.text}")
 
 		kwargs.pop("encoding", None)
 		kwargs.pop("extension", None)
