@@ -12,7 +12,7 @@ Fix for :github:issue:`51 <executablebooks/sphinx-panels>`.
 
 -----
 
-"""  # noqa: D400
+"""  # noqa: D400,RST299
 #
 #  Copyright © 2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
