@@ -32,54 +32,76 @@ General utility functions.
 #
 
 # stdlib
+import atexit
 import functools
 import re
-from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Pattern, Tuple, Type, TypeVar, cast
+import sys
+from typing import (
+		TYPE_CHECKING,
+		Any,
+		Callable,
+		Dict,
+		Iterable,
+		List,
+		Mapping,
+		Optional,
+		Pattern,
+		Set,
+		Tuple,
+		Type,
+		TypeVar,
+		cast
+		)
 
 # 3rd party
-from apeye.url import RequestsURL
+import sphinx.config
+from apeye.requests_url import RequestsURL
 from docutils.nodes import Node
 from domdf_python_tools.doctools import prettify_docstrings
+from sphinx.addnodes import desc_content
 from sphinx.application import Sphinx
 from sphinx.environment import BuildEnvironment
-from sphinx.errors import PycodeError
 from sphinx.ext.autodoc import Documenter, logger
 from sphinx.locale import __
-from sphinx.pycode import ModuleAnalyzer
 from typing_extensions import TypedDict
 
-__all__ = [
-		"make_github_url",
-		"GITHUB_COM",
-		"flag",
-		"Purger",
-		"OptionSpec",
-		"get_first_matching",
-		"escape_trailing__",
-		"code_repr",
-		"SphinxExtMetadata",
-		"SetupFunc",
-		"begin_generate",
-		"unknown_module_warning",
-		"filter_members_warning",
-		"is_namedtuple",
-		"parse_parameters",
-		"Param",
-		"typed_param_regex",
-		"untyped_param_regex",
-		"typed_flag_regex",
+__all__ = (
+		"add_nbsp_substitution",
 		"allow_subclass_add",
-		"NoMatchError",
 		"baseclass_is_private",
+		"code_repr",
+		"escape_trailing__",
+		"filter_members_warning",
+		"flag",
+		"get_first_matching",
+		"GITHUB_COM",
+		"is_namedtuple",
+		"make_github_url",
 		"metadata_add_version",
-		]
+		"NoMatchError",
+		"OptionSpec",
+		"Param",
+		"parse_parameters",
+		"Purger",
+		"SetupFunc",
+		"SphinxExtMetadata",
+		"typed_flag_regex",
+		"typed_param_regex",
+		"unknown_module_warning",
+		"untyped_param_regex",
+		"add_fallback_css_class",
+		)
 
-#: Instance of :class:`apeye.url.RequestsURL` that points to the GitHub website.
+#: Instance of :class:`apeye.requests_url.RequestsURL` that points to the GitHub website.
 GITHUB_COM: RequestsURL = RequestsURL("https://github.com")
 
 #: Type hint for the ``option_spec`` variable of Docutils directives.
 OptionSpec = Mapping[str, Callable[[str], Any]]
 
+_T = TypeVar("_T")
+
+atexit.register(GITHUB_COM.session.close)
+
 
 @functools.lru_cache()
 def make_github_url(username: str, repository: str) -> RequestsURL:
@@ -119,6 +141,8 @@ class Purger:
 
 	:param attr_name: The name of the build environment's attribute that stores the list of nodes,
 		e.g. ``all_installation_nodes``.
+
+	.. autosummary-widths:: 55/100
 	"""
 
 	def __init__(self, attr_name: str):
@@ -127,11 +151,25 @@ class Purger:
 	def __repr__(self) -> str:
 		return f"{self.__class__.__name__}({self.attr_name!r})"
 
-	def purge_nodes(self, app: Sphinx, env: BuildEnvironment, docname: str) -> None:
+	def purge_nodes(  # pragma: no cover
+			self,
+			app: Sphinx,
+			env: BuildEnvironment,
+			docname: str,
+			) -> None:
 		"""
 		Remove all redundant nodes.
 
-		:param app: The Sphinx app.
+		This function can be configured for the :event:`env-purge-doc` event:
+
+		.. code-block::
+
+			my_node_purger = Purger("all_my_node_nodes")
+
+			def setup(app: Sphinx):
+				app.connect("env-purge-doc", my_node_purger.purge_nodes)
+
+		:param app: The Sphinx application.
 		:param env: The Sphinx build environment.
 		:param docname: The name of the document to remove nodes for.
 		"""
@@ -139,12 +177,44 @@ class Purger:
 		if not hasattr(env, self.attr_name):
 			return
 
-		all_nodes = [
-				todo for todo in getattr(env, self.attr_name) if todo["docname"] != docname
-				]  # pragma: no cover
-		setattr(env, self.attr_name, all_nodes)  # pragma: no cover
+		all_nodes = [todo for todo in getattr(env, self.attr_name) if todo["docname"] != docname]
+		setattr(env, self.attr_name, all_nodes)
 
-	def add_node(self, env: BuildEnvironment, node: Node, targetnode: Node, lineno: int):
+	def get_outdated_docnames(
+			self,
+			app: Sphinx,
+			env: BuildEnvironment,
+			added: Set[str],
+			changed: Set[str],
+			removed: Set[str],
+			) -> List[str]:
+		"""
+		Returns a list of all docnames containing one or more nodes this :class:`~.Purger` is aware of.
+
+		This function can be configured for the :event:`env-get-outdated` event:
+
+		.. code-block::
+
+			my_node_purger = Purger("all_my_node_nodes")
+
+			def setup(app: Sphinx):
+				app.connect("env-get-outdated", my_node_purger.get_outdated_docnames)
+
+		.. versionadded:: 2.7.0
+
+		:param app: The Sphinx application.
+		:param env: The Sphinx build environment.
+		:param added: A set of newly added documents.
+		:param changed: A set of document names whose content has changed.
+		:param removed: A set of document names which have been removed.
+		"""
+
+		if not hasattr(env, self.attr_name):
+			return []
+
+		return list({todo["docname"] for todo in getattr(env, self.attr_name)})
+
+	def add_node(self, env: BuildEnvironment, node: Node, targetnode: Node, lineno: int) -> None:
 		"""
 		Add a node.
 
@@ -206,31 +276,26 @@ class NoMatchError(ValueError):
 	"""
 
 
-_T = TypeVar("_T")
-
-
 def get_first_matching(
 		condition: Callable[[Any], bool],
 		iterable: Iterable[_T],
-		default: _T = no_default,  # type: ignore
+		default: _T = no_default  # type: ignore[assignment]
 		) -> _T:
 	"""
 	Returns the first value in ``iterable`` that meets ``condition``, or ``default`` if none match.
 
+	.. versionadded:: 0.7.0
+
 	:param condition: The condition to evaluate.
 	:param iterable:
 	:param default: The default value to return if no values in ``iterable`` match.
-
-	:rtype:
-
-	.. versionadded:: 0.7.0
 	"""
 
 	if default is not no_default:
 		if not condition(default):
 			raise ValueError("The condition must evaluate to True for the default value.")
 
-		iterable = [*iterable, default]
+		iterable = (*iterable, default)
 
 	for match in iterable:
 		if condition(match):
@@ -243,11 +308,9 @@ def escape_trailing__(string: str) -> str:
 	"""
 	Returns the given string with trailing underscores escaped to prevent Sphinx treating them as references.
 
-	:param string:
-
-	:rtype:
-
 	.. versionadded:: 0.8.0
+
+	:param string:
 	"""
 
 	if string.endswith('_'):
@@ -259,11 +322,9 @@ def code_repr(obj: Any) -> str:
 	"""
 	Returns the repr of the given object as reStructuredText inline code.
 
-	:param obj:
-
-	:rtype:
-
 	.. versionadded:: 0.9.0
+
+	:param obj:
 	"""
 
 	return f"``{obj!r}``"
@@ -312,7 +373,7 @@ class SphinxExtMetadata(TypedDict, total=False):
 	"""
 
 
-SetupFunc = Callable[[Sphinx], Optional["SphinxExtMetadata"]]
+SetupFunc = Callable[[Sphinx], Optional[SphinxExtMetadata]]
 """
 Type annotation for Sphinx extensions' ``setup`` functions.
 
@@ -320,116 +381,32 @@ Type annotation for Sphinx extensions' ``setup`` functions.
 """
 
 
-def begin_generate(
-		documenter: Documenter,
-		real_modname: Optional[str] = None,
-		check_module: bool = False,
-		) -> Optional[str]:
-	"""
-	Boilerplate for the top of ``generate`` in :class:`sphinx.ext.autodoc.Documenter` subclasses.
-
-	:param documenter:
-	:param real_modname:
-	:param check_module:
-
-	:return: The ``sourcename``, or :py:obj:`None` if certain conditions are met,
-		to indicate that the Documenter class should exit early.
-
-	.. versionadded:: 0.2.0
-	"""
-
-	# Do not pass real_modname and use the name from the __module__
-	# attribute of the class.
-	# If a class gets imported into the module real_modname
-	# the analyzer won't find the source of the class, if
-	# it looks in real_modname.
-
-	if not documenter.parse_name():
-		# need a module to import
-		unknown_module_warning(documenter)
-		return None
-
-	# now, import the module and get object to document
-	if not documenter.import_object():
-		return None
-
-	# If there is no real module defined, figure out which to use.
-	# The real module is used in the module analyzer to look up the module
-	# where the attribute documentation would actually be found in.
-	# This is used for situations where you have a module that collects the
-	# functions and classes of internal submodules.
-	guess_modname = documenter.get_real_modname()
-	documenter.real_modname = real_modname or guess_modname
-
-	# try to also get a source code analyzer for attribute docs
-	try:
-		documenter.analyzer = ModuleAnalyzer.for_module(documenter.real_modname)
-		# parse right now, to get PycodeErrors on parsing (results will
-		# be cached anyway)
-		documenter.analyzer.find_attr_docs()
-
-	except PycodeError as err:
-		logger.debug("[autodoc] module analyzer failed: %s", err)
-		# no source file -- e.g. for builtin and C modules
-		documenter.analyzer = None  # type: ignore
-		# at least add the module.__file__ as a dependency
-		if hasattr(documenter.module, "__file__") and documenter.module.__file__:
-			documenter.directive.filename_set.add(documenter.module.__file__)
-	else:
-		documenter.directive.filename_set.add(documenter.analyzer.srcname)
-
-	if documenter.real_modname != guess_modname:
-		# Add module to dependency list if target object is defined in other module.
-		try:
-			analyzer = ModuleAnalyzer.for_module(guess_modname)
-			documenter.directive.filename_set.add(analyzer.srcname)
-		except PycodeError:
-			pass
-
-	# check __module__ of object (for members not given explicitly)
-	if check_module:
-		if not documenter.check_module():
-			return None
-
-	sourcename = documenter.get_sourcename()
-
-	# make sure that the result starts with an empty line.  This is
-	# necessary for some situations where another directive preprocesses
-	# reST and no starting newline is present
-	documenter.add_line('', sourcename)
-
-	return sourcename
-
-
 def unknown_module_warning(documenter: Documenter) -> None:
 	"""
 	Log a warning that the module to import the object from is unknown.
 
-	:param documenter:
-
-	:rtype:
-
 	.. versionadded:: 0.2.0
+
+	:param documenter:
 	"""
 
-	logger.warning(
-			__(
-					"don't know which module to import for autodocumenting "
-					'%r (try placing a "module" or "currentmodule" directive '
-					"in the document, or giving an explicit module name)"
-					) % documenter.name,
-			type="autodoc"
+	msg = __(
+			"don't know which module to import for autodocumenting %r "
+			'(try placing a "module" or "currentmodule" directive in the document, '
+			"or giving an explicit module name)"
 			)
 
+	logger.warning(msg % documenter.name, type="autodoc")
 
-def filter_members_warning(member, exception: Exception) -> None:
+
+def filter_members_warning(member: Any, exception: Exception) -> None:
 	"""
 	Log a warning when filtering members.
 
+	.. versionadded:: 0.2.0
+
 	:param member:
 	:param exception:
-
-	.. versionadded:: 0.2.0
 	"""
 
 	logger.warning(
@@ -451,7 +428,7 @@ class Param(TypedDict):
 	doc: List[str]
 
 	#: The type of the parameter.
-	type: str  # noqa A003
+	type: str  # noqa: A003  # pylint: disable=redefined-builtin
 
 
 _identifier_pattern = r"[A-Za-z_]\w*"
@@ -491,13 +468,13 @@ def parse_parameters(lines: List[str], tab_size: int = 8) -> Tuple[Dict[str, Par
 	"""
 	Parse parameters from the docstring of a class/function.
 
+	.. versionadded:: 0.8.0
+
 	:param lines: The lines of the docstring
 	:param tab_size:
 
-	:return: A dictionary mapping parameter names to their docstrings and types, a list of docstring lines that
+	:return: A mapping of parameter names to their docstrings and types, a list of docstring lines that
 		appeared before the parameters, and the list of docstring lines that appear after the parameters.
-
-	.. versionadded:: 0.8.0
 	"""
 
 	a_tab = ' ' * tab_size
@@ -508,14 +485,21 @@ def parse_parameters(lines: List[str], tab_size: int = 8) -> Tuple[Dict[str, Par
 	pre_output: List[str] = []
 	post_output: List[str] = []
 
-	def add_empty(param_name: str):
+	def add_empty(param_name: str) -> None:
 		if param_name not in params:
 			params[param_name] = {"doc": [], "type": ''}
 
 	for line in lines:
+
+		if post_output:
+			post_output.append(line)
+			continue
+
+		# pylint: disable=loop-global-usage
 		typed_m = typed_param_regex.match(line)
 		untyped_m = untyped_param_regex.match(line)
 		type_only_m = typed_flag_regex.match(line)
+		# pylint: enable=loop-global-usage
 
 		if typed_m:
 			last_arg = typed_m.group(3).strip()
@@ -546,32 +530,27 @@ def parse_parameters(lines: List[str], tab_size: int = 8) -> Tuple[Dict[str, Par
 
 def is_namedtuple(obj: Any) -> bool:
 	"""
-	Returns whether the given class is a :func:`collections.namedtuple`.
-
-	:param obj:
-
-	:rtype:
+	Returns whether the given object is a :func:`collections.namedtuple` class.
 
 	.. versionadded:: 0.8.0
+
+	:param obj:
 	"""
 
 	return isinstance(obj, type) and issubclass(obj, tuple) and hasattr(obj, "_fields")
 
 
-def allow_subclass_add(app: Sphinx, *documenters: Type[Documenter]):
+def allow_subclass_add(app: Sphinx, *documenters: Type[Documenter]) -> None:
 	"""
-	Add the given autodocumenters, but only if a subclass of it is not
-	already registered.
+	Add the given autodocumenters, but only if a subclass of it is not already registered.
 
 	This allows other libraries to extend the autodocumenters.
 
-	:param app: The Sphinx app.
-	:param documenters:
-
-	:rtype:
-
 	.. versionadded:: 0.8.0
-	"""  # noqa D400
+
+	:param app: The Sphinx application.
+	:param documenters:
+	"""
 
 	for cls in documenters:
 		existing_documenter = app.registry.documenters.get(cls.objtype)
@@ -602,7 +581,7 @@ def metadata_add_version(func: SetupFunc) -> SetupFunc:
 	"""  # noqa: D400
 
 	@functools.wraps(func)
-	def wrapper(app: Sphinx):
+	def wrapper(app: Sphinx) -> SphinxExtMetadata:
 
 		# this package
 		from sphinx_toolbox import __version__
@@ -612,3 +591,246 @@ def metadata_add_version(func: SetupFunc) -> SetupFunc:
 		return ret
 
 	return wrapper
+
+
+def add_nbsp_substitution(config: sphinx.config.Config) -> None:
+	"""
+	Adds the ``|nbsp|`` substitution directive to the reStructuredText prolog.
+
+	.. versionadded:: 2.1.0
+
+	:param config:
+	"""
+
+	nbsp_sub = ".. |nbsp| unicode:: 0xA0\n   :trim:"
+
+	if not config.rst_prolog:
+		config.rst_prolog = ''  # type: ignore[attr-defined]
+
+	if nbsp_sub not in config.rst_prolog:
+		config.rst_prolog = '\n'.join([config.rst_prolog, '', nbsp_sub])  # type: ignore[attr-defined]
+
+
+_OBJTYPES_CSS_FALLBACKS = {
+		"namedtuple": "class",
+		"protocol": "class",
+		"typeddict": "class",
+		}
+
+# From https://github.com/mansenfranzen/autodoc_pydantic/pull/86/files
+# MIT Licensed
+# Copyright (c) 2021 Franz Wöllert
+
+
+def add_fallback_css_class(
+		objtypes_css_fallbacks: Dict[str, str]
+		) -> Callable[[Sphinx, str, str, desc_content], None]:
+	"""
+	Registers a transform which will edit the CSS classes of documented objects based on their ``objtype``.
+
+	:param objtypes_css_fallbacks: A mapping of Sphinx objtypes to the CSS class which should be added to them.
+		The class is usually the ``objtype`` attribute from the documenter's parent class.
+
+	.. versionadded:: 2.16.0
+
+	Used as follows:
+
+	.. code-block:: python
+
+		app.connect("object-description-transform", add_fallback_css_class({"typeddict": "class"}))
+
+	This will apply the transformation to documented objects with the ``typeddict`` CSS class
+	by adding the ``class`` CSS class.
+
+	:param objtypes_css_fallbacks:
+	"""
+
+	def func(
+			app: Sphinx,
+			domain: str,
+			objtype: str,
+			contentnode: desc_content,
+			) -> None:
+
+		if objtype not in objtypes_css_fallbacks:
+			return
+
+		classes = contentnode.parent.attributes["classes"]
+
+		# for older sphinx versions, add objtype explicitly
+		if sphinx.version_info < (3, 6):
+			classes.append(objtype)
+
+		idx = classes.index(objtype)
+		fallback = objtypes_css_fallbacks[objtype]
+		classes.insert(idx, fallback)
+
+	return func
+
+
+if TYPE_CHECKING:
+
+	class Config(sphinx.config.Config):
+		project: Any
+		author: Any
+		project_copyright: str
+		copyright: str  # noqa: A003  # pylint: disable=redefined-builtin
+		version: Any
+		release: Any
+		today: Any
+		today_fmt: str
+		language: str
+		locale_dirs: Any
+		figure_language_filename: str
+		gettext_allow_fuzzy_translations: Any
+		master_doc: Any
+		root_doc: Any
+		source_suffix: Any
+		source_encoding: Any
+		exclude_patterns: Any
+		default_role: str
+		add_function_parentheses: Any
+		add_module_names: Any
+		trim_footnote_reference_space: Any
+		show_authors: Any
+		pygments_style: Any
+		highlight_language: Any
+		highlight_options: Any
+		templates_path: Any
+		template_bridge: Any
+		keep_warnings: Any
+		suppress_warnings: Any
+		modindex_common_prefix: Any
+		rst_epilog: str
+		rst_prolog: str
+		trim_doctest_flags: Any
+		primary_domain: Any
+		needs_sphinx: str
+		needs_extensions: Any
+		manpages_url: Any
+		nitpicky: Any
+		nitpick_ignore: Any
+		nitpick_ignore_regex: Any
+		numfig: Any
+		numfig_secnum_depth: Any
+		numfig_format: Any
+		math_number_all: Any
+		math_eqref_format: str
+		math_numfig: Any
+		tls_verify: Any
+		tls_cacerts: Any
+		user_agent: str
+		smartquotes: Any
+		smartquotes_action: Any
+		smartquotes_excludes: Any
+		config_values: Dict[str, Tuple]
+		overrides: Dict
+		values: Dict[str, Tuple]
+		setup: Optional[Callable]
+		extensions: List[str]
+		latex_engine: str
+		latex_documents: Any
+		latex_logo: str
+		latex_appendices: Any
+		latex_use_latex_multicolumn: Any
+		latex_use_xindy: bool
+		latex_toplevel_sectioning: str
+		latex_domain_indices: List
+		latex_show_urls: Any
+		latex_show_pagerefs: Any
+		latex_elements: Any
+		latex_additional_files: Any
+		latex_theme: str
+		latex_theme_options: Any
+		latex_theme_path: Any
+		latex_docclass: Any
+		html_theme: Any
+		html_theme_path: Any
+		html_theme_options: Any
+		html_title: str
+		html_short_title: Any
+		html_style: str
+		html_logo: str
+		html_favicon: str
+		html_css_files: Any
+		html_js_files: Any
+		html_static_path: Any
+		html_extra_path: Any
+		html_last_updated_fmt: str
+		html_sidebars: Any
+		html_additional_pages: Any
+		html_domain_indices: List
+		html_add_permalinks: Any
+		html_permalinks: Any
+		html_permalinks_icon: Any
+		html_use_index: Any
+		html_split_index: Any
+		html_copy_source: Any
+		html_show_sourcelink: Any
+		html_sourcelink_suffix: Any
+		html_use_opensearch: Any
+		html_file_suffix: str
+		html_link_suffix: str
+		html_show_copyright: Any
+		html_show_search_summary: Any
+		html_show_sphinx: Any
+		html_context: Any
+		html_output_encoding: Any
+		html_compact_lists: Any
+		html_secnumber_suffix: Any
+		html_search_language: str
+		html_search_options: Any
+		html_search_scorer: Any
+		html_scaled_image_link: Any
+		html_baseurl: Any
+		html_codeblock_linenos_style: str
+		html_math_renderer: Any
+		html4_writer: Any
+else:
+	Config = sphinx.config.Config
+
+if sphinx.version_info[0] >= 5 or TYPE_CHECKING:  # pragma no cover
+
+	class RemovedInSphinx50Warning(RuntimeError):
+		pass
+
+	def prepare_docstring(s: str, ignore: Optional[int] = None, tabsize: int = 8) -> List[str]:
+
+		if ignore is None:
+			ignore = 1
+		else:
+			raise TypeError("The 'ignore' argument to prepare_docstring() was removed in Sphinx 5.0")
+
+		lines = s.expandtabs(tabsize).splitlines()
+
+		# Find minimum indentation of any non-blank lines after ignored lines.
+		margin = sys.maxsize
+		for line in lines[ignore:]:
+			content = len(line.lstrip())
+			if content:
+				indent = len(line) - content
+				margin = min(margin, indent)
+
+		# Remove indentation from ignored lines.
+		for i in range(ignore):
+			if i < len(lines):
+				lines[i] = lines[i].lstrip()
+
+		if margin < sys.maxsize:
+			for i in range(ignore, len(lines)):
+				lines[i] = lines[i][margin:]
+
+		# Remove any leading blank lines.
+		while lines and not lines[0]:
+			lines.pop(0)
+
+		# make sure there is an empty line at the end
+		if lines and lines[-1]:
+			lines.append('')
+
+		return lines
+
+else:
+	# 3rd party
+	from sphinx.deprecation import RemovedInSphinx50Warning  # type: ignore[attr-defined,no-redef]  # noqa: F401
+	from sphinx.util.docstrings import prepare_docstring  # noqa: F401
