@@ -4,13 +4,14 @@
 """
 Sphinx extension to create links to Wikipedia articles.
 
+.. versionadded:: 0.2.0
 .. extensions:: sphinx_toolbox.wikipedia
 
-.. versionadded:: 0.2.0
 
+Configuration
+--------------
 
-Usage
-------
+.. latex:vspace:: -5px
 
 .. confval:: wikipedia_lang
 	:type: :class:`str`
@@ -22,14 +23,19 @@ Usage
 	.. versionadded:: 0.2.0
 
 
+Usage
+------
+
+.. latex:vspace:: -5px
+
 .. rst:role:: wikipedia
 
-	Shows a link to the given article on Wikipedia.
+	Role which shows a link to the given article on Wikipedia.
 
 	The title and language can be customised.
 
 
-	**Example**
+	:bold-title:`Example`
 
 	.. rest-example::
 
@@ -37,17 +43,20 @@ Usage
 
 		:wikipedia:`mythical creature <Sphinx>`
 
-		:wikipedia:`:zh:斯芬克斯`
-
 		:wikipedia:`Answer to the Ultimate Question of Life, the Universe, and Everything <:de:42 (Antwort)>`
 
+	.. only:: html
+
+		.. rest-example::
+
+			:wikipedia:`:zh:斯芬克斯`
 
 
 API Reference
 ----------------
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -80,18 +89,23 @@ from urllib.parse import quote
 
 # 3rd party
 from apeye.url import URL
-from docutils import nodes, utils
+from docutils import nodes
 from docutils.nodes import system_message
 from docutils.parsers.rst.states import Inliner
 from sphinx.application import Sphinx
 from sphinx.util.nodes import split_explicit_title
 
 # this package
-from sphinx_toolbox.utils import SphinxExtMetadata
+from sphinx_toolbox.utils import SphinxExtMetadata, metadata_add_version
 
-__all__ = ["make_wikipedia_link", "setup"]
+__all__ = ("make_wikipedia_link", "setup")
 
 base_url = "https://%s.wikipedia.org/wiki"
+_wiki_lang_re = re.compile(":(.*?):(.*)")
+
+
+def _get_wikipedia_lang(inliner: Inliner) -> str:  # pragma: no cover
+	return inliner.document.settings.env.config.wikipedia_lang
 
 
 def make_wikipedia_link(
@@ -121,42 +135,35 @@ def make_wikipedia_link(
 	:return: A list containing the created node, and a list containing any messages generated during the function.
 	"""
 
-	env = inliner.document.settings.env
-	lang = env.config.wikipedia_lang
-
-	text = utils.unescape(text)
+	text = nodes.unescape(text)
 	has_explicit, title, target = split_explicit_title(text)
 
-	m = re.match(r":(.*?):(.*)", target)
+	m = _wiki_lang_re.match(target)
+
 	if m:
 		lang, target = m.groups()
 		if not has_explicit:
 			title = target
+	else:
+		lang = _get_wikipedia_lang(inliner)
 
-	lang_url = URL(base_url % lang)
-
-	ref = lang_url / quote(target.replace(' ', '_'), safe='')
+	ref = URL(base_url % lang) / quote(target.replace(' ', '_'), safe='')
 
 	node = nodes.reference(rawtext, title, refuri=str(ref), **options)
 	return [node], []
 
 
+@metadata_add_version
 def setup(app: Sphinx) -> SphinxExtMetadata:
 	"""
 	Setup :mod:`sphinx_toolbox.wikipedia`.
 
-	:param app: The Sphinx app.
-
 	.. versionadded:: 1.0.0
-	"""
 
-	# this package
-	from sphinx_toolbox import __version__
+	:param app: The Sphinx application.
+	"""
 
 	app.add_role("wikipedia", make_wikipedia_link)
 	app.add_config_value("wikipedia_lang", "en", "env", [str])
 
-	return {
-			"version": __version__,
-			"parallel_read_safe": True,
-			}
+	return {"parallel_read_safe": True}
